unit carte;

interface

uses
  Graphics, Types,
  reference;

type
  TImageCarte = class
  private
    FSourceFile: string;
    function GetWidth: integer; virtual; abstract;
    function GetHeight : integer; virtual; abstract;
    function GetPixels(X, Y : integer) : TColor; virtual; abstract;
  public
    property SourceFile : string read FSourceFile;
    property Width : integer read GetWidth;
    property Height : integer read GetHeight;
    property Pixels[X, Y: Integer]: TColor read GetPixels;
    constructor Create(FileName : string); virtual;
    procedure Draw(var Support : TBitmap; X, Y, Ratio : integer); virtual; abstract;
    procedure DrawPoint(Support: TCanvas; Center: TPoint; Text: string; Color: TColor);
    procedure DrawRefPoint(Support : TCanvas; X, Y, Ratio : integer;
                                 RefPoint : TReferencePoint; Color : TColor);
    procedure DrawSegment(Support: TCanvas; PtStart, PtEnd: TPoint; Color: TColor);
  end;

  TImageCarteBasic = class(TImageCarte)
  // for classical pictures like bmp, tif, jpeg, fully loaded in memory
  private
    BitMap : TBitMap;
    function GetWidth: integer; override;
    function GetHeight : integer; override;
    function GetPixels(X, Y : integer) : TColor; override;
  public
    constructor Create(FileName : string); override;
    destructor Destroy; override;
    procedure Draw(var Support : TBitmap; X, Y, Ratio : integer); override;
  end;

implementation

uses
  SysUtils, jpeg,
  GraphicEx,
  drawing_const;

// GraphicEx from http://www.soft-gems.net/  

{ TImageCarte }

constructor TImageCarte.Create(FileName: string);
begin
  FSourceFile:=FileName;
end;

procedure TImageCarte.DrawPoint(Support: TCanvas; Center: TPoint; Text: string; Color: TColor);
begin
  with Support do
  begin
    Pen.Color:=Color;
    Brush.Style:=bsSolid;
    Brush.Color:=Color;
    Pen.Width:=1;
    Ellipse(Center.X-PtSize, Center.Y-PtSize, Center.X+PtSize, Center.Y+PtSize);
    if Text<>'' then
    begin
      Brush.Style:=bsClear;
      Font.Name:=PtFontName;
      Font.Size:=PtFontSize;
      Font.Color:=Color;
      TextOut(Center.X+TextShift, Center.Y, Text);
    end;
  end;
end;

procedure TImageCarte.DrawRefPoint(Support: TCanvas; X, Y,
  Ratio: integer; RefPoint: TReferencePoint; Color: TColor);
var
  Center : TPoint;
begin
  Center.X:=RefPoint.XPixel div Ratio - X;
  Center.Y:=RefPoint.YPixel div Ratio - Y;
  DrawPoint(Support, Center, RefPoint.Name, Color);
  // triangle surrounding the point
  with Support do
  begin
    Pen.Color:=Color;
    Pen.Width:=2;
    MoveTo(Center.X, Center.Y - round(TriangleSize * 0.577));
    LineTo(Center.X + TriangleSize div 2, Center.Y + round(TriangleSize * 0.289));
    LineTo(Center.X - TriangleSize div 2, Center.Y + round(TriangleSize * 0.289));
    LineTo(Center.X, Center.Y - round(TriangleSize * 0.577));
  end;
end;

procedure TImageCarte.DrawSegment(Support: TCanvas; PtStart, PtEnd: TPoint; Color: TColor);
begin
  with Support do
  begin
    Pen.Color:=Color;
    Brush.Style:=bsSolid;
    Brush.Color:=Color;
    Pen.Width:=2;
    MoveTo(PtStart.X, PtStart.Y);
    LineTo(PtEnd.X, PtEnd.Y);
  end;
end;

{ TImageCarteBasic }

constructor TImageCarteBasic.Create(FileName: string);
var
  JpegImage : TJpegImage;
  Ext : string;
begin
  inherited;
  Ext := UpperCase(ExtractFileExt(FileName));
  if Ext='.BMP' then
  begin
    Bitmap:=TBitmap.Create;
    Bitmap.LoadFromFile(FileName);
  end  
  else
    if Ext='.JPG' then
    begin
      Bitmap:=TBitmap.Create;
      JPegImage := TJpegImage.Create;
      try
        JPegImage.LoadFromFile(FileName);
        Bitmap.Assign(JpegImage);
      finally
        JPegImage.Free;
      end;  
    end
    else
    begin
      if Ext='.PNG' then
      begin
        Bitmap := TPNGGraphic.Create;
        Bitmap.LoadFromFile(FileName);
      end
      else
      begin
        if Ext='.TIF' then
        begin
          Bitmap := TTIFFGraphic.Create;
          Bitmap.LoadFromFile(FileName);
        end
        else
          raise Exception.Create('Format image inconnu') at @TImageCarteBasic.Create;
      end;
    end;
  if Bitmap.PixelFormat<>pf24bit then
    Bitmap.PixelFormat:=pf24bit;
end;

procedure TImageCarteBasic.Draw(var Support: TBitmap; X, Y,
  Ratio: integer);
var
  Haut, Bas, Gauche, Droite : integer;
  I, J, XSource, YSource : integer;
//  LSource, LDest : PByteArray;
  LSource, LDest : pointer;
begin
  assert(Bitmap.PixelFormat=pf24bit,'Wrong bitmap format for Bitmap in TImageCarteBasic.Draw');
  assert(Support.PixelFormat=pf24bit,'Wrong bitmap format for Support in TImageCarteBasic.Draw');
  inherited;
  if Ratio=1 then
  begin  // 100 %
//    Support.Canvas.CopyRect(Rect(0,0,Support.Width-1, Support.Height-1),Bitmap.Canvas,Rect(X,Y,X+Support.Width-1, Y+Support.Height-1));
    if X>0 then
      Gauche:=0
    else
      Gauche:=-X;
    if Y>0 then
      Haut:=0
    else
      Haut:=-Y;
    if X+Support.Width<Bitmap.Width then
      Droite:=Support.Width
    else
      Droite:=Bitmap.Width-X;
    if Y+Support.Height<Bitmap.Height then
      Bas:=Support.Height
    else
      Bas:=Bitmap.Height-Y;

    for I:=0 to Haut-1 do
    begin
      LDest:=Support.ScanLine[I];
//      Fillchar(LDest^[0], Support.Width*3, 128);
      Fillchar(LDest^, Support.Width*3, 128);
    end;
    for I:=Haut to Bas-1 do
    begin
      LDest:=Support.ScanLine[I];
      LSource:=Bitmap.ScanLine[I+Y];
      Fillchar(LDest^, Support.Width*3, 128);
//      Fillchar(LDest^[0], Support.Width*3, 128);
      inc(integer(LSource),3*(X+Gauche));
      inc(integer(LDest),3*Gauche);
      Move(LSource^, LDest^,(Droite-Gauche)*3);
//      Move(LSource^[3*(X+Gauche)], LDest^[Gauche*3],(Droite-Gauche)*3);

    end;
    for I:=Bas to Support.Height-1 do
    begin
      LDest:=Support.ScanLine[I];
      Fillchar(LDest^, Support.Width*3, 128);
    end;
  end
  else
  begin  // <100%
    for I:=0 to Support.Height-1 do
    begin
      LDest:=Support.ScanLine[I];
      YSource:=(I+Y)*Ratio;
      if (YSource<0) or (YSource>=Bitmap.Height) then
        fillchar(LDest^,Support.Width*3,128)
      else
      begin
        LSource:=Bitmap.ScanLine[YSource];
            inc(integer(LSource),3*X*Ratio);
        for J:=0 to Support.Width-1 do
        begin
          XSource:=(J+X)*Ratio;
          if (XSource<0) or (XSource>=Bitmap.Width) then
          begin
            byte(LDest^) := 128; inc(integer(LDest));
            byte(LDest^) := 128; inc(integer(LDest));
            byte(LDest^) := 128; inc(integer(LDest));
//            LDest^[3*J]:=128;
//            LDest^[3*J+1]:=128;
//            LDest^[3*J+2]:=128;
          end
          else
          begin
            byte(LDest^) := byte(LSource^); inc(integer(LDest)); inc(integer(LSource));
            byte(LDest^) := byte(LSource^); inc(integer(LDest)); inc(integer(LSource));
            byte(LDest^) := byte(LSource^); inc(integer(LDest)); inc(integer(LSource));
            inc(integer(LSource),3*(Ratio-1));
//            LDest^[3*J]:=LSource^[3*XSource];
//            LDest^[3*J+1]:=LSource^[3*XSource+1];
//            LDest^[3*J+2]:=LSource^[3*XSource+2];
          end;
        end;
      end;
    end;
  end;
end;

function TImageCarteBasic.GetHeight: integer;
begin
  result:=Bitmap.Height;
end;

function TImageCarteBasic.GetWidth: integer;
begin
  result:=Bitmap.Width;
end;

function TImageCarteBasic.GetPixels(X, Y : integer) : TColor;
begin
  result := Bitmap.Canvas.Pixels[X, Y];
end;

destructor TImageCarteBasic.Destroy;
begin
  if BitMap <>nil then
    BitMap.Free;
  inherited;
end;

end.
